@@ -1,4 +1,4 @@
-
+import six
 
 class iRODSMeta(object):
 
@@ -95,6 +95,12 @@ class iRODSMetaCollection(object):
         """
         Returns a list of iRODSMeta associated with a given key
         """
+        if six.PY2:
+            if isinstance(key, unicode):
+                key = key.encode('utf8')
+        else:
+            if isinstance(key, bytes):
+                key = key.decode('utf8')
         if not isinstance(key, str):
             raise TypeError
         return [m for m in self._meta if m.name == key]
